//! A complex number x = a + ib
class Complex 
{
  public float|int r, i ; 
  
  //!We can create a complex given Its real and imaginary part, or only with one argument: an array.
  void create( float|int real, float|int imaginari )
  {
    
    r = real ;
    i = imaginari ;
  }
  
  //!Returns the real part of the complex number.
  float|int realPart()
  {
    return r ;
  }
  
  //!Returns the imaginary part of the complex number.
  float|int imPart()
  {
    return i ;
  }
  
  Complex `+(Complex a )
  {
    Complex resultat = Complex( r + a->realPart(), i + a->imPart() );
    return resultat ;
  }
  
  //Multiplicació de dos complexos
  Complex `*( Complex a )
  {
    float real = a->realPart();
    float im = a->imPart();
    float nreal = real * r - i*im ;
    float nim = real*i + r * im ;
    Complex resultat = Complex( nreal, nim );
    return resultat ;
  }
  
  //Multipliació d'un Complex per un escalar.
  Complex `%( float|int a )
  {
    Complex resultat = Complex ( a * r , a * i );
    return resultat ;
  }
  
  //! a - b
  Complex `-( Complex a  )
  {
    Complex resultat = Complex( r - a->realPart() , i - a->imPart() );
    return resultat ;
  }
  
  //! We can cast a Complex number to a string in a form Real + i * Imaginary
  string cast( string what )
  {
    switch( what )
      {
      case "string" :
	return r + " + " + i + " i " ;
	break;
      }
  }
  
  //!Returns the module of a complex number.
  float module()
  {
    return sqrt(r*r + i*i) ;
  }
  
  //!Given a + ib, returns a -ib
  Complex conjugate( )
  {
    Complex resultat = Complex(r, -i);
    return resultat ;
  }
  
  Complex `/ (Complex a )
  {
    Complex conjugat = a->conjugate();
    float modul = a->module();
    Complex resultat = `*( conjugat );
    return resultat % (1.0 / (modul * modul) ) ;
  }
}


//!The imaginary unit.
Complex i = Complex(0,1);

//!Exponential of a Complex number.
Complex Exp( Complex a )
{
  float|int r = a->realPart();
  float|int i = a->imPart();
  float ex = exp(r);
  Complex resultat = Complex( ex * cos(i), ex * sin(i ) );
  return resultat ;
}



int main ()
{
  Complex cmp = Complex(1.0, 2.5);
  float pi = 4.0 * atan(1.0);
  write("We have the complex number " + cmp->realPart() + " + " + cmp->imPart() + " * i \n");
  Complex resultat = cmp + i ;
  write("The result is " + resultat->realPart() + " + " + resultat->imPart() + " *i \n");
  Complex r_mult = cmp * i ;
  write("Ths result is " + (string) r_mult + "\n");
  write ( "i/i = " + (string ) (i / i) + "\n") ;
  write("Exp(i pi) = " +  (string) Exp( i % pi ) + "\n" ) ;
  write( " i / 10 = " + (string) (i / Complex(10, 0 ) ) + "\n");
  return 0;
}


