import .Complex ;
import .LinearAlgebra ;


float pi = 4.0 * atan(1.0);

//! Does the discrete fourier transformation of some data.
array DiscreteFourierTransformation ( array data, int max )
{
  int N = sizeof(data);
  array vectoret = ({ });
  for ( int n = 0 ; n < max ; ++n )
    {
      Complex suma = Complex(0.0, 0.0) ;
      for ( int k = 0 ; k < N ; ++k )
	{
	  float hk = data[k] ;
	  suma +=   Exp (i % ( 2 * pi  * k * n / N )) % hk ;
	}
      vectoret += ({ suma }) ;
    }
  return vectoret ;
}


//! Does the inverse discrete fourier transformation of data.
array InverseDiscreteFourierTransformation ( array data, int max )
{
  int N = sizeof(data);
  array vectoret = ({ });
  Complex divisor = Complex(N , 0);
  for ( int n = 0 ; n < max ; ++n )
    {
      Complex suma = Complex(0.0, 0.0) ;
      for ( int k = 0 ; k < N ; ++k )
	{
	  Complex hk = data[k] ;
	  suma +=   Exp (i % ( -2 * pi  * k * n / N )) * hk  ;
	}
      vectoret += ({ suma / divisor}) ;
    }
  return vectoret ;
}

//! Power spectrum of data. This first does a discreteFourierTransformation of data, them computes the powerSpectrum.
array PowerSpectrum( array data, int max )
{
  array resultat = ({ });
  array transformada = DiscreteFourierTransformation ( data, max );
  foreach ( transformada, Complex item )
    {
      resultat += ({ pow(item->module() , 2) }) ;
    }
  return resultat ;
}

int main ()
{
  array data = ({ 1.0, 2.0, 3.0 , 4.0 , 5.0 , 6.0 , 7.0 , 8.0 , 9.0 , 10.0 });
  write("Raw data\n");
  foreach( data, float item )
    {
      write(  item + "\n" ) ;
    }
  array resultat = DiscreteFourierTransformation( data, 10 );
  write("Discrete fourier transformation of the data: \n");
  foreach( resultat, Complex item )
    {
      write( (string) item + "\n" ) ;
    }
  resultat = InverseDiscreteFourierTransformation( resultat, 10 );
  write("Inverse fourier transformation \n");
  foreach( resultat, Complex item )
    {
      write( (string) item + "\n" ) ;
    }
  resultat = PowerSpectrum(data , 10 );
  write("Power espectrum \n");
  foreach( resultat, float item )
    {
      write(  item + "\n" ) ;
    }
  return 0 ;
}
 
